# BEASTRunner

THIS SCRIPT automates conducting multiple runs of BEAST1 or BEAST2 (Drummond et al. 2012; Bouckaert et al. 2014) XML input files on a remote supercomputing cluster that uses SLURM resource management with PBS wrappers, or a PBS resource management system.

The code starts from a single working directory on the user's local machine, which contains one or multiple XML input files with extension '\*run.xml' (where * is any set of alphanumeric characters separated possibly by underscores but no spaces). These files are identified and run through the BEASTRunner pipeline, which involves four steps, as follows: 1) copy each XML file four times to create 5 separate run XML files; 2) make directories for runs (1 per XML) and create a shell script with the name 'beast\_pbs.sh' that is specific to the input and can be used to submit job to supercomputer and move this PBS shell script into the corresponding folder; 3) create a batch submission file (PBS format) and move it and all run folders to the desired working directory on the supercomputer; 4) execute the batch submission file on the supercomputer so that all jobs are submitted to the supercomputer queue.

Like other software programs such as BPP, G-PhoCS, and GARLI, some information that is used by BEASTRunner.sh is fed to the program by culling the data from an external configuration file, named 'beast\_runner.cfg'. There are six entries that users can supply in this file. However, three of these are essential for running BEAST using the BEASTRunner script, including: ssh user account info, the path to the parent directory for BEAST runs on the supercomputer, and the user's email address. Users must fill this information in and save a new version of the .cfg file in the working directory on their local machine prior to calling the program. Only then can the user call the program by opening a terminal window, typing "./BEASTRunner.sh", and pressing return.

It is assumed that BEAST1 (e.g. v1.8.3) or BEAST2 (e.g. 2.4++) is installed on the supercomputer, and that the user can provide absolute paths to the software in the configuration file. During generation of the BEAST submission scripts (STEP #2 above), there are options within the shell script template area of the BEASTRunner code (on Lines 268-288) that could (and probably should) be edited by the user, but which provide an illustration of how to load prerequisite Java and Beagle library modules on a supercomputing cluster that uses modules. However, especially when many different versions are available, or modules are unavailable on your supercomputer, it will be more convenient to supply the whole path to BEAST executables instead of using module code like this. 

## POTENTIAL ERRORS / ISSUES TO AVOID:

Users will need to be especially mindful of the following potential issues, to avoid them and keep BEASTRunner from failing on the first go-throughs:

1. Like most other scripts within PIrANHA, BEASTRunner assumes that the user has set
   up passwordless ssh acces. See notes within the script, and in the PIrANHA README
   file for links and information on how to set this up **PRIOR TO RUNNING**.

2. The supercomputer that BEASTRunner was initially developed to work on uses modules
   to load different versions of BEAST and java/java runtime environment on which
   BEAST software depends. Make sure to modify the module calls in the script to match 
   your cluster and needs, or remove them as needed. 
       - If you run BEASTRunner on the BYU FSL supercomputer, it will almost run out 
         of the box. You will only need to point the BEAST paths to your local installs
         using the configuration file, or edit the module code if needed.
       - Users of other supercomputing clusters will likely need to make more substantial
         changes to the code before running (e.g. removing module code during shell 
         script generation).

3. If your supercomputing cluster does not have a PBS wrapper, then the shell scripts
   generated by BEASTRunner to submit each unique job/XML file will not be correctly
   read and interpreted by your batch management software. In this case, you will
   most likely need to modify the text echoed to file on Lines 87-129 to take the 
   format of a SLURM sbatch script. In future versions of BEASTRunner, I will extend
   the code to accomodate both TORQUE/PBS resource management and SLURM Workload
   Manager, so that you can simply direct the script to use one or the other. 

## REFERENCES

- Bouckaert R, Heled J, Künert D, Vaughan TG, Wu CH, Xie D, Suchard MA, Rambaut A, Drummond AJ 
(2014) BEAST2: a software platform for Bayesian evolutionary analysis. PLoS Computational 
Biology, 10, e1003537.
- Drummond AJ, Suchard MA, Xie D, Rambaut A (2012) Bayesian phylogenetics with BEAUti and the 
BEAST 1.7. Molecular Biology and Evolution, 29, 1969-1973.

August 22, 2017
Justin C. Bagley, Richmond, VA, USA
