#!/bin/sh

##########################################################################################
#  __  o  __   __   __  |__   __                                                         #
# |__) | |  ' (__( |  ) |  ) (__(                                                        # 
# |                                                                                      #
#                          getBootTrees.sh v0.1.0, August 2017                           #
#  SHELL SCRIPT FOR COLLATING AND ORGANIZING BOOTSTRAP TREES FROM RAxML RUNS ON MULTIPLE #
#  INPUT TREE FILES                                                                      #
#  Copyright (c)2017 Justinc C. Bagley, Virginia Commonwealth University, Richmond, VA,  #
#  USA; Universidade de Brasília, Brasília, DF, Brazil. See README and license on GitHub #
#  (http://github.com/justincbagley) for further info. Last update: August 20, 2017.     #
#  For questions, please email jcbagley@vcu.edu.                                         #
##########################################################################################

echo "
##########################################################################################
#                          getBootTrees.sh v0.1.0, August 2017                           #
##########################################################################################
"

######################################## START ###########################################
echo "INFO      | $(date) | Starting getBootTrees.sh script... "

	################################## getBootTrees.sh #######################################
	echo "INFO      | $(date) |          Organizing bootstrap trees and making final output file containing all trees... "
	echo "INFO      | $(date) |          Making list of ML bootstrap trees generated by RAxML... "

	ls **/RAxML_bootstrap.raxml_out > bootTrees.list

	##--Assign bootstrap tree list to variable
	MY_BOOT_TREE_LIST="$(cat ./bootTrees.list)"

	############ ORGANIZE BOOTSTRAP TREES INTO ONE LOCATION
	##--Place all inferred bootstrap tree files into a single "bootstrap_trees" folder in 
	##--working directory. However, all the boot tree files have the same name. So, in order
	##--to do this, we have to give each boot tree file a name that matches the corresponding
	##--run folder, i.e. locus. We can rename each file right after downloading it.
	mkdir ./bootstrap_trees

	echo "INFO      | $(date) |          Copying *ALL* ML bootstrap trees to 'bootstrap_trees' folder in current directory for post-processing..."
	(
		for j in ${MY_BOOT_TREE_LIST}; do
			echo "$j"
			cp "$j" ./bootstrap_trees/
			MY_LOCUS_NAME="$(echo $j | sed 's/\/[A-Za-z.\_\-]*//g')"
			cp ./bootstrap_trees/RAxML_bootstrap.raxml_out ./bootstrap_trees/"$MY_LOCUS_NAME"_RAxML_boot.tre
			rm ./bootstrap_trees/RAxML_bootstrap.raxml_out
		done
	)

	echo "INFO      | $(date) |          Making final output file containing best ML trees from all runs/loci..."
	(
		for k in ./bootstrap_trees/*; do
			echo "$k"
			cat "$k" >> ./boottrees.tre
		done
	)

	echo "INFO      | $(date) |          Making final list of ML bootstrap trees in bootstrap_trees directory..."
	ls ./bootstrap_trees/*.tre > final_bootTrees.list
 
echo "INFO      | $(date) | Done collating ML bootstrap trees from all RAxML runs (independent sub-folders of pwd) run using getBootTrees.sh."
echo "INFO      | $(date) | Bye.
"
#
#
#
######################################### END ############################################

exit 0
